@@ -1,11 +1,12 @@
 from naomi.eeprom import NaomiEEPRom, NaomiEEPRomException
 from naomi.generic_patch import force_freeplay, force_no_attract_sound
-from naomi.rom import NaomiRom, NaomiRomRegionEnum, NaomiExecutable, NaomiRomSection, NaomiRomException
+from naomi.rom import NaomiRom, NaomiRomRegionEnum, NaomiRomVersionEnum, NaomiExecutable, NaomiRomSection, NaomiRomException
 from naomi.rom_patch import NaomiSettingsPatcher, NaomiSettingsPatcherException, NaomiSettingsTypeEnum, get_default_trojan, add_or_update_trojan, add_or_update_section
 
 __all__ = [
     "NaomiRom",
     "NaomiRomRegionEnum",
+    "NaomiRomVersionEnum",
     "NaomiExecutable",
     "NaomiRomSection",
     "NaomiRomException",
